<?php

namespace Drupal\Tests\node_edit_redirect\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Tests that the node edit redirect works.
 *
 * @group node_edit_redirect 
 */
class NodeEditRedirectTest extends BrowserTestBase {
  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'node_edit_redirect',
    'locale',
    'content_translation',
  ];

  /**
   * Use the standard profile.
   *
   * @var string
   */
  protected $profile = 'standard';

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
   parent::setUp();
   $admin = $this->drupalCreateUser([], NULL, TRUE);
   $this->drupalLogin($admin);
   // Create FR.
   $this->drupalPostForm('admin/config/regional/language/add', [
     'predefined_langcode' => 'fr',
   ], 'Add language');
   // Set prefixes to en and fr.
   $this->drupalPostForm('admin/config/regional/language/detection/url', [
     'prefix[en]' => 'en',
     'prefix[fr]' => 'fr',
   ], 'Save configuration');
   // Set up URL and language selection page methods.
   $this->drupalPostForm('admin/config/regional/language/detection', [
     'language_interface[enabled][language-selection-page]' => 1,
     'language_interface[enabled][language-url]' => 1,
   ], 'Save settings');
   // Turn on content translation for pages.
   $this->drupalPostform('admin/structure/types/manage/page', ['language_configuration[content_translation]' => 1], 'Save content type');
  }

  /**
   * Test that the redirect works.
   */
  public function testRedirect() {
   // Create Basic page in Spanish.
   $node_title = $this->randomName();
   $node_body =  $this->randomName();
   $node = $this->createPage($node_title, $node_body, 'es');

   $this->drupalLogin($this->admin_user);
   // Visit the Italian page for a Spanish node.
   $this->drupalGet('it/node/' . $node->nid . '/edit');
   $this->assertResponse(200);
   $headers = $this->drupalGetHeaders(TRUE);
   list(, $status) = explode(' ', $headers [0][':status'], 3);
   $this->assertEqual($status, 302, 'Expected response code was sent.');
   $languages = language_list();

   // Assert that we redirected to the Spanish edit page.
   $this->assertEqual($this->getUrl(), url('node/' . $node->nid . '/edit', ['absolute' => TRUE, 'language' => $languages['es']]));
   $this->assertNotEqual($this->getUrl(), url('node/' . $node->nid . '/edit', ['absolute' => TRUE, 'language' => $languages['it']]));
  }

  /**
   * Installs the specified language, or enables it if it is already installed.
   *
   * @param $language_code
   *   The language code to check.
   */
  protected function addLanguage($language_code) {
   // Check to make sure that language has not already been installed.
   $this->drupalGet('admin/config/regional/language');

   if (strpos($this->drupalGetContent(), 'enabled[' . $language_code . ']') === FALSE) {
    // Doesn't have language installed so add it.
    $edit = [];
    $edit['langcode'] = $language_code;
    $this->drupalPost('admin/config/regional/language/add', $edit, t('Add language'));

    // Make sure we are not using a stale list.
    drupal_static_reset('language_list');
    $languages = language_list('language');
    $this->assertTrue(array_key_exists($language_code, $languages), 'Language was installed successfully.');

    if (array_key_exists($language_code, $languages)) {
     $this->assertRaw(t('The language %language has been created and can now be used. More information is available on the <a href="@locale-help">help screen</a>.', ['%language' => $languages[$language_code]->name, '@locale-help' => url('admin/help/locale')]), 'Language has been created.');
    }
   }
   elseif ($this->xpath('//input[@type="checkbox" and @name=:name and @checked="checked"]', [':name' => 'enabled[' . $language_code . ']'])) {
    // It's installed and enabled. No need to do anything.
    $this->assertTrue(true, 'Language [' . $language_code . '] already installed and enabled.');
   }
   else {
    // It's installed but not enabled. Enable it.
    $this->assertTrue(true, 'Language [' . $language_code . '] already installed.');
    $this->drupalPost(NULL, ['enabled[' . $language_code . ']' => TRUE], t('Save configuration'));
    $this->assertRaw(t('Configuration saved.'), 'Language successfully enabled.');
   }
  }

  /**
   * Creates a "Basic page" in the specified language.
   *
   * @param $title
   *   The title of a basic page in the specified language.
   * @param $body
   *   The body of a basic page in the specified language.
   * @param $language
   *   (optional) Language code.
   *
   * @return
   *   A node object.
   */
  protected function createPage($title, $body, $language = NULL) {
   $edit = [];
   $langcode = LANGUAGE_NONE;
   $edit["title"] = $title;
   $edit["body[$langcode][0][value]"] = $body;
   if (!empty($language)) {
    $edit['language'] = $language;
   }
   $this->drupalPost('node/add/page', $edit, t('Save'));
   $this->assertRaw(t('Basic page %title has been created.', ['%title' => $title]), 'Basic page created.');

   // Check to make sure the node was created.
   $node = $this->drupalGetNodeByTitle($title);
   $this->assertTrue($node, 'Node found in database.');

   return $node;
  }

  
 }
